@@ -351,7 +351,7 @@ pub enum IntEncoding {
     None,
     /// Signed.
     Signed,
-    /// It's a c_char.
+    /// It's a `c_char`.
     Char,
     /// It's a bool.
     Bool,
@@ -619,15 +619,30 @@ gen_collection_concrete_type! {
         /// The name of this enum variant.
         pub name: Option<&'btf OsStr>,
         /// The numeric value of this enum variant.
-        pub value: i32,
+        pub value: i64,
     }
 
-    |btf, member| EnumMember {
-        name: btf.name_at(member.name_off),
-        value: member.val,
+    |btf, member, signed| {
+        EnumMember {
+            name: btf.name_at(member.name_off),
+            value: if signed {
+                member.val.into()
+            } else {
+                u32::from_ne_bytes(member.val.to_ne_bytes()).into()
+            }
+        }
     }
 }
 
+impl Enum<'_> {
+    /// Check whether the enum is signed or not.
+    #[inline]
+    pub fn is_signed(&self) -> bool {
+        self.kind_flag()
+    }
+}
+
+
 // Fwd
 gen_fieldless_concrete_type! {
     /// A forward declared C type.
@@ -792,7 +807,7 @@ gen_concrete_type! {
 impl DeclTag<'_> {
     /// The component index is present only when the tag points to a struct/union member or a
     /// function argument.
-    /// And component_idx indicates which member or argument, this decl tag refers to.
+    /// And `component_idx` indicates which member or argument, this decl tag refers to.
     #[inline]
     pub fn component_index(&self) -> Option<u32> {
         self.ptr.component_idx.try_into().ok()
@@ -819,19 +834,33 @@ gen_collection_concrete_type! {
         /// The name of this enum variant.
         pub name: Option<&'btf OsStr>,
         /// The numeric value of this enum variant.
-        pub value: u64,
+        pub value: i128,
     }
 
-    |btf, member| Enum64Member {
+    |btf, member, signed| Enum64Member {
         name: btf.name_at(member.name_off),
         value: {
             let hi: u64 = member.val_hi32.into();
             let lo: u64 = member.val_lo32.into();
-            hi << 32 | lo
+            let val = (hi << 32) | lo;
+            if signed {
+                i64::from_ne_bytes(val.to_ne_bytes()).into()
+            } else {
+                val.into()
+            }
         },
     }
 }
 
+impl Enum64<'_> {
+    /// Check whether the enum is signed or not.
+    #[inline]
+    pub fn is_signed(&self) -> bool {
+        self.kind_flag()
+    }
+}
+
+
 /// A macro that allows matching on the type of a [`BtfType`] as if it was an enum.
 ///
 /// Each pattern can be of two types.
@@ -861,7 +890,7 @@ gen_collection_concrete_type! {
 ///     }
 /// ```
 ///
-/// NonBinding.
+/// Non-binding.
 ///
 /// ```compile_fail
 ///     BtfKind::Int => {
